# -*- coding: utf-8 -*-

"""
Sample script to show how to work with the ppcardhandler library.

1. Make use of ppapiaccess library (dependency) to fetch user (get_me())
   and available projects (get_projects()).
2. Make use of Project class in ppcardhandler library to get interface
   to boards and cards and fetch/modify data.

If you like to get access to more data/functionality in the
Projectplace.com API then you will find useful stuff in core.py
where the Board/Card sepcific calls are made to the Projectplace.com
API. Documentation is available at http://api.projectplace.com. 

Originally by C.Dalid, 2013-05-15
2013-08-xx Extended with support for label and description on card.
"""




def make_changes(creds):
    """
    Get specific Board on specific Project.
    Create/get specific Card and make changes to it.

    @param creds: Credentials
    @type creds: {...} (see ppapiaccess.connect for further info)
    """
    from ppapiaccess import connect
    from ppcardhandler import Project

    PROJECTNAME = '<name of project to work with>'
    BOARDNAME = '<name of board to work with>'
    CARDNAME = '<name of card to work with>' # Will be created if it doesn't exist
    STATUSNAME = '<name of status to set>'   # Does not have to be exact, will use first status where name starts with this value
    LABELNAME = '<name of label to set>'   # Does not have to be exact, will use first label where name starts with this value
    DESCRIPTION = '<description to set>'
    

    ## Initialize connection to API (primary credentials)
    connect(credentials=creds)
    
    
    # 1) What project to work with
    proj =  Project(name=PROJECTNAME, id=None, data={})
    print proj      # Show the rough print/representation of a Project object

    # 2) What board to work with
    print
    print '**************************'
    board = proj.board(name=BOARDNAME)
    print board     # Show the rough print/representation of a Board object

    # 3) What card to work with
    card = board.card(name=CARDNAME, create=True)
    print card      # Show the rough print/representation of a Card object
    
    print
    print 'Properties of the card:'
    
    # 4) Card Status
    print ' - Status of card:', card.status.name
    if STATUSNAME:  # Update status
        status = board.status(STATUSNAME, exactmatchonly=False)
        card.set_status(status)
        print ' - New status of card:',card.status.name

    # 5) Card Label
    print ' - Label of card:', card.label.name
    if LABELNAME: # Update label
        label = board.label(LABELNAME, exactmatchonly=False)
        card.set_label(label)
        print ' - New label of card:',card.label.name


    # 6) Card description
    print ' - Description of card:', card.description
    if DESCRIPTION: # Update description
        card.set_description(DESCRIPTION)
        print ' - New description of card:', card.description

def main(creds):
    """
    Authenticate with Projectplace.com API and present
    some basic info about the user.

    @param creds: Credentials
    @type creds: {...} (see ppapiaccess.connect for further info)
    """

    ## Import core / basic utils
    from ppapiaccess import connect, get_me, get_projects
    from ppcardhandler import Project

    ## Initialize connection to API (primary credentials)
    connect(credentials=creds)
    
    print
    print "Info fetched via API regarding current user (based on credentials):"
    data = get_me()
    project_list = get_projects()
    print " - Name: %s %s"%(data['first_name'],data['last_name'])
    print " - Email:",data['email']
    print " - Number of available projects:",len(project_list)
    print

    projects = [Project(name=proj['name'], id=proj['id'], data={}) for proj in project_list]
    print "Will go through first 2 projects and list available boards in each..."
    for proj in projects[:2]:
        print
        print 'BOARDS IN PROJECT "%s" (%d boards):'%(proj.name, len(proj.boards))
        for board_id, board in proj.boards.items():
            print ' - "%s" (%s).'%(board.name, board.id)
            print '    * Number of cards on board: %d'%(len(board.cards))
            statuses = [status.name for status in board.statuses]
            print '    * Statuses: %s'%','.join(statuses)
            labels = [label.name for label in board.labels]
            print '    * Labels: %s'%','.join(labels)
            
        
    


if __name__ == '__main__':

    
    ## TODO: Support provision of credentials via arguments?
    ##       (i.e. Individual credentials, path to conf file)

    # Make use of default configuration file. This can be replaced
    # by your own configuration file, view existing for syntax.
    confpath = ''
    if not confpath:
        from ppapiaccess import default_confpath
        confpath = default_confpath

    # Extract credentials from config file    
    from ppapiaccess.credentials import credentials_from_cfg
    creds = credentials_from_cfg(filepath=confpath)

    # Values for "oauth_token_secret" and "oauth_token" are not
    # available by default in the default configuration file (default_confpath).
    # These values refer to user specific credentials and must be obtained
    # through a OAuth authentication process.
    # If you haven't specified them in config file or here in this file
    # then the process will automatically trigger such OAuth authentication
    # process when you run this script. Afterwards you can take the generated
    # token & secret and either put them here in the file or in the config
    # file to make data accessible for all scripts that used the ppapiaccess lib.

    #creds['oauth_token_secret'] = ''
    #creds['oauth_token'] = ''
    

    ## Call main() for basic example
    main(creds)
    

    ## Example of fetching specific Project and make change to Card
    ## You need to specify project name etc. to run this.
    #make_changes(creds)

    import sys
    print
    print 'Script (%s) finished.'%sys.argv[0]
